@@ -2,7 +2,7 @@
 Name:    py416.filesystem
 Author:  Ezio416
 Created: 2022-08-16
-Updated: 2022-09-09
+Updated: 2022-09-12
 
 Functions for file system manipulation
 '''
@@ -42,6 +42,10 @@ class File():
     def exists(self) -> bool:
         return os.path.exists(self.path)
 
+    @property
+    def isroot(self) -> bool:
+        return self.path == str(self.parent)
+
     @property
     def mtime(self) -> float:
         return os.path.getmtime(self.path)
@@ -52,7 +56,7 @@ class File():
 
     @property
     def name(self) -> str:
-        return os.path.basename(self.path) if not self.root else self.path
+        return os.path.basename(self.path) if not self.isroot else self.path
 
     @property
     def parent(self) -> str:
@@ -63,8 +67,8 @@ class File():
         return splitpath(self.path)
 
     @property
-    def root(self) -> bool:
-        return self.path == str(self.parent)
+    def root(self) -> str:
+        return self.parts[0]
 
     @property
     def size(self) -> int:
@@ -72,7 +76,7 @@ class File():
 
     @property
     def stem(self) -> str:
-        return self.name.split('.')[0]
+        return self.name if self.isdir else self.name.split('.')[0]
 
     @property
     def suffix(self) -> str:
@@ -129,6 +133,40 @@ def cd(dir:str='..') -> bool:
     except Exception:
         return False
 
+def checkzip(path:str) -> bool:
+    '''
+    - Imports: `py7zr`: `path` ends with .7z
+    - Checks if an archive file (.7z or .zip) exists and is valid
+    - Deletes if invalid or incomplete
+    - Input: `path` (`str`): path to the archive file
+    - Return:
+        - `True`: file exists and is valid
+        - `False`: file doesn't exist, possibly because we deleted it due to corruption
+    '''
+    if gettype(path) != 'str':
+        raise TypeError('input must be a string')
+    try:
+        if path.endswith('.7z'):
+            from py7zr import exceptions, SevenZipFile
+            try:
+                with SevenZipFile(path, 'r') as z:
+                    pass
+                return True
+            except exceptions.Bad7zFile:
+                os.remove(path)
+                return False
+        elif path.endswith('.zip'):
+            from zipfile import BadZipFile, ZipFile
+            try:
+                with ZipFile(path) as z:
+                    pass
+                return True
+            except BadZipFile:
+                os.remove(path)
+                return False
+    except FileNotFoundError:
+        return False
+
 def forslash(path:str) -> str:
     '''
     - Replaces `\\` in paths with `/`
@@ -249,6 +287,16 @@ def parent(path:str) -> str:
     except NameError:
         return getcwd()
 
+def pathjoin(*parts) -> str:
+    '''
+    - Imitator for `os.path.join()`
+    - Joins parts of a path into a string path
+    - Input: `parts` (iterable): directories/file to join together
+    - Return: `str` with path (formatted with `/`)
+    '''
+    parts_ = [str(item).replace('/', '').replace('\\', '') for item in unpack(parts)]
+    return '/' if parts_ == [''] else '/'.join(parts_)
+
 def realpath(path:str) -> str:
     '''
     - Wrapper for `os.path.realpath()`
@@ -263,7 +311,7 @@ def realpath(path:str) -> str:
 def rename(path:str, name:str) -> str:
     '''
     - Wrapper for `os.rename()`
-    - Renames file
+    - Renames file without moving it
     - Input:
         - `path` (`str`): path to file/directory to be renamed
         - `name` (`str`): new basename for file (not path)
@@ -307,9 +355,62 @@ def splitpath(path:str) -> list:
     if gettype(path) != 'str':
         raise TypeError('Input must be a string')
     result = forslash(path).split('/')
-    if result[0] == '':
-        result[0] = '/'
+    result[0] = f'{result[0]}/' # root
     if result[1] == '':
         return [result[0]]
     return result
 
+def unzip(path:str, delete:bool=False) -> int:
+    '''
+    - Unzips archive files of type: (.7z, .gz, .rar, .tar, .zip)
+    - Input:
+        - `path` (`str`): path to archive file
+        - `delete` (`bool`): whether to delete archive after unzipping
+            - Default: `False`
+    - Return:
+        - `0` for success
+        - `1` for delete failure
+        - `2` for unzip failure
+        - `-1` if nothing was attempted
+    '''
+    if gettype(path) != 'str':
+        raise TypeError('Input must be a string')
+    delete = bool(delete)
+    fparent = parent(path)
+    if path.endswith('.7z'):
+        try:
+            from py7zr import unpack_7zarchive
+            unpack_7zarchive(path, fparent)
+            if delete:
+                os.remove(path)
+        except Exception:
+            return 2
+    elif path.endswith(tuple(['.gz', '.rar', '.tar', '.zip'])):
+        try:
+            sh.unpack_archive(path, fparent)
+            if delete:
+                os.remove(path)
+        except Exception:
+            return 2
+    else:
+        return -1
+    return 0
+
+def unzipdir(dir:str='.') -> int:
+    '''
+    - Unzips all archives in a directory until it is unable to continue
+    - Input: `dir` (`str`): directory from which to unzip archives
+        - Default: current working directory
+    - Return: `int` of unzipped archives
+    '''
+    unzipped = 0
+    while True:
+        unzipped_thisrun = 0
+        for file in listdir(dir, dirs=False):
+            if not unzip(file, delete=True):
+                unzipped += 1
+                unzipped_thisrun += 1
+        if not unzipped_thisrun:
+            break
+    return unzipped
+
